@@ -35,12 +35,9 @@
 #ifdef _WIN32
 #include <windows.h>
 #include <process.h>
-typedef HANDLE js_thread_t;
 #else
 #include <dirent.h>
-#include <pthread.h>
 #include <unistd.h>
-typedef pthread_t js_thread_t;
 #endif
 
 #include "cutils.h"
@@ -405,13 +402,22 @@ static int add_test_file(const char *filename)
 
 static void find_test_files(const char *path);
 
+static bool ispathsep(int c)
+{
+    return c == '/' || c == '\\';
+}
+
 static void consider_test_file(const char *path, const char *name, int is_dir)
 {
+    size_t pathlen;
     char s[1024];
 
     if (str_equal(name, ".") || str_equal(name, ".."))
         return;
-    snprintf(s, sizeof(s), "%s/%s", path, name);
+    pathlen = strlen(path);
+    while (pathlen > 0 && ispathsep(path[pathlen-1]))
+        pathlen--;
+    snprintf(s, sizeof(s), "%.*s/%s", (int)pathlen, path, name);
     if (is_dir)
         find_test_files(s);
     else
@@ -521,49 +527,6 @@ static JSValue js_evalScript_262(JSContext *ctx, JSValueConst this_val,
     return ret;
 }
 
-static void start_thread(js_thread_t *thrd, void *(*start)(void *), void *arg)
-{
-    // musl libc gives threads 80 kb stacks, much smaller than
-    // JS_DEFAULT_STACK_SIZE (1 MB)
-    static const unsigned stacksize = 2 << 20; // 2 MB, glibc default
-#ifdef _WIN32
-    HANDLE h, cp;
-
-    cp = GetCurrentProcess();
-    h = (HANDLE)_beginthread((void (*)(void *))(void *)start, stacksize, arg);
-    if (!h)
-        fatal(1, "_beginthread error");
-    // _endthread() automatically closes the handle but we want to wait on
-    // it so make a copy. Race-y for very short-lived threads. Can be solved
-    // by switching to _beginthreadex(CREATE_SUSPENDED) but means changing
-    // |start| from __cdecl to __stdcall.
-    if (!DuplicateHandle(cp, h, cp, thrd, 0, false, DUPLICATE_SAME_ACCESS))
-        fatal(1, "DuplicateHandle error");
-#else
-    pthread_attr_t attr;
-
-    if (pthread_attr_init(&attr))
-        fatal(1, "pthread_attr_init");
-    if (pthread_attr_setstacksize(&attr, stacksize))
-        fatal(1, "pthread_attr_setstacksize");
-    if (pthread_create(thrd, &attr, start, arg))
-        fatal(1, "pthread_create error");
-    pthread_attr_destroy(&attr);
-#endif
-}
-
-static void join_thread(js_thread_t thrd)
-{
-#ifdef _WIN32
-    if (WaitForSingleObject(thrd, INFINITE))
-        fatal(1, "WaitForSingleObject error");
-    CloseHandle(thrd);
-#else
-    if (pthread_join(thrd, NULL))
-        fatal(1, "pthread_join error");
-#endif
-}
-
 static long cpu_count(void)
 {
 #ifdef _WIN32
@@ -612,7 +575,7 @@ typedef struct {
 static JSValue add_helpers1(JSContext *ctx);
 static void add_helpers(JSContext *ctx);
 
-static void *agent_start(void *arg)
+static void agent_start(void *arg)
 {
     ThreadLocalStorage *tls;
     Test262Agent *agent;
@@ -688,7 +651,6 @@ static void *agent_start(void *arg)
 
     JS_FreeContext(ctx);
     JS_FreeRuntime(rt);
-    return NULL;
 }
 
 static JSValue js_agent_start(JSContext *ctx, JSValueConst this_val,
@@ -712,7 +674,7 @@ static JSValue js_agent_start(JSContext *ctx, JSValueConst this_val,
     agent->tls = tls;
     JS_FreeCString(ctx, script);
     list_add_tail(&agent->link, &tls->agent_list);
-    start_thread(&agent->tid, agent_start, agent);
+    js_thread_create(&agent->tid, agent_start, agent, /*flags*/0);
     return JS_UNDEFINED;
 }
 
@@ -724,7 +686,7 @@ static void js_agent_free(JSContext *ctx)
 
     list_for_each_safe(el, el1, &tls->agent_list) {
         agent = list_entry(el, Test262Agent, link);
-        join_thread(agent->tid);
+        js_thread_join(agent->tid);
         JS_FreeValue(ctx, agent->broadcast_sab);
         list_del(&agent->link);
         free(agent);
@@ -1754,7 +1716,8 @@ JSContext *JS_NewCustomContext(JSRuntime *rt)
 int run_test_buf(ThreadLocalStorage *tls, const char *filename, char *harness,
                  namelist_t *ip, char *buf, size_t buf_len,
                  const char* error_type, int eval_flags, bool is_negative,
-                 bool is_async, bool can_block, int *msec)
+                 bool is_async, bool can_block, bool track_promise_rejections,
+                 int *msec)
 {
     JSRuntime *rt;
     JSContext *ctx;
@@ -1764,6 +1727,7 @@ int run_test_buf(ThreadLocalStorage *tls, const char *filename, char *harness,
     if (rt == NULL) {
         fatal(1, "JS_NewRuntime failure");
     }
+    JS_SetDumpFlags(rt, JS_DUMP_LEAKS);
     JS_SetRuntimeOpaque(rt, tls);
     js_std_init_handlers(rt);
     ctx = JS_NewCustomContext(rt);
@@ -1778,6 +1742,9 @@ int run_test_buf(ThreadLocalStorage *tls, const char *filename, char *harness,
     /* loader for ES6 modules */
     JS_SetModuleLoaderFunc(rt, NULL, js_module_loader_test, (void *) filename);
 
+    if (track_promise_rejections)
+        JS_SetHostPromiseRejectionTracker(rt, js_std_promise_rejection_tracker, NULL);
+
     add_helpers(ctx);
 
     for (i = 0; i < ip->count; i++) {
@@ -1824,6 +1791,7 @@ int run_test(ThreadLocalStorage *tls, const char *filename, int *msec)
     int ret, eval_flags, use_strict, use_nostrict;
     bool is_negative, is_nostrict, is_onlystrict, is_async, is_module, skip;
     bool detect_module = true;
+    bool track_promise_rejections = false;
     bool can_block;
     namelist_t include_list = { 0 }, *ip = &include_list;
 
@@ -1882,6 +1850,9 @@ int run_test(ThreadLocalStorage *tls, const char *filename, int *msec)
                 else if (str_equal(option, "qjs:no-detect-module")) {
                     detect_module = false;
                 }
+                else if (str_equal(option, "qjs:track-promise-rejections")) {
+                    track_promise_rejections = true;
+                }
                 else if (str_equal(option, "module")) {
                     is_module = true;
                     skip |= skip_module;
@@ -1976,12 +1947,13 @@ int run_test(ThreadLocalStorage *tls, const char *filename, int *msec)
         if (use_nostrict) {
             ret = run_test_buf(tls, filename, harness, ip, buf, buf_len,
                                error_type, eval_flags, is_negative, is_async,
-                               can_block, msec);
+                               can_block, track_promise_rejections, msec);
         }
         if (use_strict) {
             ret |= run_test_buf(tls, filename, harness, ip, buf, buf_len,
                                 error_type, eval_flags | JS_EVAL_FLAG_STRICT,
-                                is_negative, is_async, can_block, msec);
+                                is_negative, is_async, can_block,
+                                track_promise_rejections, msec);
         }
     }
     namelist_free(&include_list);
@@ -2009,6 +1981,7 @@ int run_test262_harness_test(ThreadLocalStorage *tls, const char *filename,
     if (rt == NULL) {
         fatal(1, "JS_NewRuntime failure");
     }
+    JS_SetDumpFlags(rt, JS_DUMP_LEAKS);
     JS_SetRuntimeOpaque(rt, tls);
     ctx = JS_NewContext(rt);
     if (ctx == NULL) {
@@ -2074,7 +2047,7 @@ int run_test262_harness_test(ThreadLocalStorage *tls, const char *filename,
 
 clock_t last_clock;
 
-void *show_progress(void *unused) {
+void show_progress(void *unused) {
     int interval = 1000*1000*1000 / 4; // 250 ms
 
     js_mutex_lock(&progress_mutex);
@@ -2087,7 +2060,6 @@ void *show_progress(void *unused) {
         fflush(stderr);
     }
     js_mutex_unlock(&progress_mutex);
-    return NULL;
 }
 
 enum { INCLUDE, EXCLUDE, SKIP };
@@ -2103,7 +2075,7 @@ int include_exclude_or_skip(int i) // naming is hard...
     return INCLUDE;
 }
 
-void *run_test_dir_list(void *arg)
+void run_test_dir_list(void *arg)
 {
     ThreadLocalStorage tls_s, *tls = &tls_s;
     const char *p;
@@ -2120,7 +2092,6 @@ void *run_test_dir_list(void *arg)
         if (verbose > 1 || (slow_test_threshold && msec >= slow_test_threshold))
             fprintf(stderr, "%s (%d ms)\n", p, msec);
     }
-    return NULL;
 }
 
 void help(void)
@@ -2295,15 +2266,17 @@ int main(int argc, char **argv)
         }
         js_cond_init(&progress_cond);
         js_mutex_init(&progress_mutex);
-        start_thread(&progress_thread, show_progress, NULL);
+        js_thread_create(&progress_thread, show_progress, NULL, /*flags*/0);
+        for (i = 0; i < nthreads; i++) {
+            js_thread_create(&threads[i], run_test_dir_list,
+                             (void *)(uintptr_t)i, /*flags*/0);
+        }
         for (i = 0; i < nthreads; i++)
-            start_thread(&threads[i], run_test_dir_list, (void *)(uintptr_t)i);
-        for (i = 0; i < nthreads; i++)
-            join_thread(threads[i]);
+            js_thread_join(threads[i]);
         js_mutex_lock(&progress_mutex);
         js_cond_signal(&progress_cond);
         js_mutex_unlock(&progress_mutex);
-        join_thread(progress_thread);
+        js_thread_join(progress_thread);
         js_mutex_destroy(&progress_mutex);
         js_cond_destroy(&progress_cond);
     } else {
